/* ---------------------------------------------------------------------- */
/*                                DEMO5.C                                 */
/*                                                                        */
/*      Resolucao de equacoes polinomiais pelo metodo de Bairstow (1)     */
/*                                                                        */
/*                    (C) 1993, Lenimar Nunes de Andrade                  */
/*                           CCENDM03 @ BRUFPB.BITNET                     */
/*                                                                        */
/*       Veja o  arquivo  SOLVEQ.DOC  para  maiores informacoes sobre as  */
/* as funcoes e estruturas usados.                                        */
/*                                                                        */
/*        Para compilar este programa, use o TCC do Turbo C 2.0 e digite  */
/*                      [C:\] TCC -G -Z demo5 solveq.lib                  */
/*        Para salvar em disco a solucao digite                           */
/*                      [C:\] DEMO5 > NomeArq.Ext                         */
/* ---------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>

#include "solveq.h"

main () {

  /* --- Variaveis globais --- */
  PolinomioReal f, g, f_copia;
  polinomio f_comp;
  int i = 0, n1, n2, RaizesEncontradas = 0;
  float p = 1, q = 1, re1, re2, im1, im2;
  complexo *raizes;

  /* --- Leitura do grau e dos coeficientes do polinomio p --- */
  LePolinomioReal(&f);

  /* --- Define f_comp como sendo uma copia complexa de f --- */
  f_comp.grau = f.grau;
  InicializaPolinomio(&f_comp);
  ConvertePolinomioRealComplexo(f, &f_comp);

  /* --- Aloca memoria para os coeficientes dos polinomios g e f_copia --- */
  g.grau = f.grau;
  f_copia.grau = f.grau;
  InicializaPolinomioReal(&g);
  InicializaPolinomioReal(&f_copia);

  /* --- Define g e f_copia como sendo copias de f --- */
  CopiaPolinomioReal(&g, f);
  CopiaPolinomioReal(&f_copia, f);

  /* --- Aloca memoria para as raizes --- */
  raizes = (complexo *) malloc(f.grau*sizeof(complexo));
  fprintf(stderr, "\nResolvendo...");

  /* --- Resolucao da equacao --- */

  /*
   *  Raizes nulas devem ser consideradas separadamente
   */

  while (f.grau && Bairstow(f, &p, &q, &n1, 1E-7, 50)) {
    if (!Bairstow(f_copia, &p, &q, &n2, 1E-12, 50)) break;
    SegundoGrauReal(p, q, &re1, &im1, &re2, &im2);
    raizes[i].a = re1; raizes[i].b = im1; i++;
    raizes[i].a = re2; raizes[i].b = im2; i++;
    DividePolSegGrau(f, p, q, &g);
    CopiaPolinomioReal(&f, g);
    RaizesEncontradas += 2;

    if (f.grau == 2) {
      /* --- Resolucao de uma equacao do 2o. grau --- */
      p = COEF(f, 1); q = COEF(f, 0);
      Bairstow(f_copia, &p, &q, &n1, 1E-12, 100);
      SegundoGrauReal(p, q, &re1, &im1, &re2, &im2);
      raizes[i].a = re1; raizes[i].b = im1; i++;
      raizes[i].a = re2; raizes[i].b = im2;
      f.grau = 0; RaizesEncontradas += 2;
    }
    /* --- Resolucao de uma equacao do 1o. grau --- */
    else if (f.grau == 1) {
      re1 = -COEF(f, 0)/COEF(f, 1); n1 = n2 = 0; im1 = 0;
      raizes[i].a = re1; raizes[i].b = im1;
      f.grau = 0; RaizesEncontradas++;
    }
  }

  clrscr();
  puts("RESOLUCAO DE EQUACOES POLINOMIAIS USANDO O METODO DE BAIRSTOW\n");

  printf("Grau = %d\n", f_comp.grau);
  puts("\nCoeficientes:\n");

  /* --- Mostra os coeficientes de p_copia em um campo de largura 80. Se o
         valor for fracionario, entao ele sera' mostrado com 6 decimais --- */
  MostraPolinomio(f_comp, 80, 6);

  puts("\n\nRaizes:");
  printf("\n   #        parte real      parte imaginaria   ");
  printf("\n---------------------------------------------");
  for (i = 0; i < RaizesEncontradas; i++)
    printf("\n  %2d    %16.10Lf  %16.10Lf", i+1, raizes[i].a, raizes[i].b);
  printf("\n---------------------------------------------\n");

  /* --- Libera a memoria alocada dinamicamente --- */
  free(raizes);
  LiberaPolinomioReal(&f);
  LiberaPolinomioReal(&f_copia);
  LiberaPolinomioReal(&g);
  LiberaPolinomio(&f_comp);

}

/*** FIM DE "DEMO5.C" ***/
