/* ---------------------------------------------------------------------- */
/*                                DEMO6.C                                 */
/*                                                                        */
/*   Resolucao de equacoes polinomiais reais pelo metodo de Bairstow (2)  */
/*                                                                        */
/*                   (C) 1993, Lenimar Nunes de Andrade                   */
/*                         CCENDM03 @ BRUFPB.BITNET                       */
/*                                                                        */
/*       Veja  o  arquivo  SOLVEQ.DOC  para maiores informacoes sobre as  */
/* funcoes e estruturas usados.                                           */
/*                                                                        */
/*        Para compilar este programa, use o TCC do Turbo C 2.0 e digite  */
/*                     [C:\] TCC -G -Z demo6 solveq.lib                   */
/*        Para salvar em disco a solucao digite                           */
/*                      [C:\] DEMO6 > NomeArq.Ext                         */
/* ---------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>

#include "solveq.h"

main () {

  /* --- Variaveis globais --- */
  solucao raizes;
  polinomio p, p_copia;
  complexo *v, *zinv;
  char expo[100], base[100];
  int i, ini;

  /* --- Aproximacao inicial para p0 e q0, usados no calculo da primeira raiz
         de p --- */
  raizes.p0 = 1; raizes.q0 = 1;

  /* --- Constantes que limitam a quantidade de tentativas --- */
  raizes.nit1 = 50; raizes.nit2 = 50; raizes.lim = 100;

  /* --- Erros usados na aproximacao inicial e no refinamento das raizes --- */
  raizes.erro1 = 1E-7; raizes.erro2 = 1E-12;

  /* --- Leitura do grau (p.grau) e dos coeficientes (p.coef[i]) do
         polinomio p. Todos os coeficientes de p devem ser reais. --- */
  LePolinomio(&p);
  for (i = 0; i <= p.grau; i++) p.coef[i].b = 0;

  /* --- Inicializa o polinomio p_copia e define-o como uma copia de p --- */
  p_copia.grau = p.grau;
  InicializaPolinomio(&p_copia);
  CopiaPolinomio(&p_copia, p);

  clrscr();

  /* --- Mostra na tela o polinomio p no formato usual usando um campo de
     largura 8, variavel 'x', e coeficientes decimais com 6 casas --- */
  puts("RESOLUCAO DE EQUACOES POLINOMIAIS USANDO O METODO DE BAIRSTOW\n");
  puts("Equacao:");
  ini = p.grau;
  do {
    ini = FormataPolinomio(p, 78, ini, 'x', base, expo, 6);
    puts("");
    puts(expo);
    puts(base);
  } while (ini >= -1);
  puts("");

  /* --- Aloca memoria para as raizes, iteracoes, v e zinv --- */
  raizes.raiz = (complexo *) malloc(p.grau*sizeof(complexo));
  raizes.iteracao1 = (int *) malloc(p.grau*sizeof(int));
  raizes.iteracao2 = (int *) malloc(p.grau*sizeof(int));
  raizes.metodo = (char *) malloc(p.grau*sizeof(char));
  v = (complexo *) malloc(p.grau*sizeof(complexo));
  zinv = (complexo *) malloc(p.grau*sizeof(complexo));

  raizes.linha = wherey();
  gotoxy(1, raizes.linha);
  raizes.encontradas = 0;

  /* --- Determina as raizes do polinomio p_copia, guardando o resultado
         em raizes. Raizes nulas devem ser consideradas separadamente --- */
  ResolveEqMetBtw(p_copia, &raizes);

  /* --- Testa as raizes encontradas --- */
  CopiaPolinomio(&p_copia, p);
  gotoxy(1, raizes.linha); clreol();
  fprintf(stderr, "Testando as raizes encontradas...");
  TestaRaizes(p_copia, raizes, v, zinv);

  /* --- Mostra na tela os resultados encontrados --- */
  gotoxy(1, raizes.linha); clreol();

  puts("Raizes:");
  printf("\n   #        parte real      parte imaginaria    teste ");
  printf("\n----------------------------------------------------------");
  for (i = 0; i < raizes.encontradas; i++) {
    printf("\n  %2d    %16.10Lf  %16.10Lf", i+1, raizes.raiz[i].a, raizes.raiz[i].b);
    if ((AbsComp(v[i]) < 1E-12) || (AbsComp(zinv[i]) < 1E-12))
      printf("        Ok");
    else
      printf("  Raiz suspeita");
  }
  printf("\n----------------------------------------------------------\n");

  /* --- Libera a memoria ocupada por raizes.raiz, p.coef, p_copia.coef,
         v e zinv --- */
  free(raizes.raiz);
  free(raizes.iteracao1);
  free(raizes.iteracao2);
  free(raizes.metodo);
  free(p.coef);
  free(p_copia.coef);
  free(v);
  free(zinv);
}

/*** FIM DE "DEMO6.C" ***/

